MaybeOwnedMut

Enum MaybeOwnedMut 

pub enum MaybeOwnedMut<'a, T>
where T: 'a,
{ Owned(T), Borrowed(&'a mut T), }
This type is basically the same as MaybeOwned, but works with mutable references.

Note that while you can se MaybeOwned as a alternative implementation for a Cow (Copy-On-Write) type this isn’t really the case for MaybeOwnedMut as changing it will potentially change the source through the given &mut reference. For example the transitive add assign (+=) implementation for MaybeOwned does (need to) convert the given instance into a owned variant before using += on the contained type. But for MaybeOwnedMut it can directly use += on the &mut contained in the Borrowed variant!

Variants



Owned(T)

owns T



Borrowed(&'a mut T)

has a reference to T

Implementations

Source

impl<T> MaybeOwnedMut<'_, T>

pub fn is_owned(&self) -> bool

Returns true if the data is owned else false.

Source

impl<T> MaybeOwnedMut<'_, T>
where T: Clone,

pub fn into_owned(self) -> T

Return the contained data in it’s owned form.

If it’s borrowed this will clone it.

pub fn make_owned(&mut self) -> &mut T

Internally converts the type into it’s owned variant.

Conversion from a reference to the owned variant is done by cloning.

This returns a &mut T and as such can be used to “unconditionally” get an &mut T. Be aware that while this works with both MaybeOwned and MaybeOwnedMut it also converts it to an owned variant in both cases. So while it’s the best way to get a &mut T for MaybeOwned for MaybeOwnedMut it’s preferable to use as_mut from AsMut.

Example
use maybe_owned::MaybeOwned;

#[derive(Clone, Debug, PartialEq, Eq)]
struct PseudoBigData(u8);

let data = PseudoBigData(12);

let mut maybe: MaybeOwned<PseudoBigData> = (&data).into();
assert_eq!(false, maybe.is_owned());

{
    let reference = maybe.make_owned();
    assert_eq!(&mut PseudoBigData(12), reference);
}
assert!(maybe.is_owned());